/*
 * Copyright (c) 2008 Lu, Chao-Ming (Tetralet).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "config.h"

// default settings
gchar* cfgfile = "lilyterm.rc";
gchar* page_name = "Terminal";
gchar* foreground_color = "white";
gchar* background_color = "black";
gchar* font_name = "Monospace 15";
gint screen_with = 80;
gint screen_height = 25;
gboolean transparent_background = FALSE;
gdouble background_saturation = 0.2;
gchar* word_chars = "-A-Za-z0-9_$.+!*(),;:@&=?/~#%[]";
gint scrollback_lines = 1024;
gboolean show_menu = FALSE;
gchar* default_locale;
gchar* locales_list = "EUC-JP GB2312 Big5";
gchar** supported_locales;
VteTerminal* current_vtebox;

struct KeyValue pagekeys[KEYS];

void get_user_settings()
{

	// New Page
	pagekeys[0].name = "new_tab_key";
	pagekeys[0].value = "Ctrl T";
	// Close Page
	pagekeys[1].name = "close_tab_key";
	pagekeys[1].value = "Ctrl W";
	// Pre Page
	pagekeys[2].name = g_strdup("pre_tab_key");
	pagekeys[2].value = g_strdup("Ctrl Page_Up");
	// Next Page
	pagekeys[3].name = g_strdup("next_tab_key");
	pagekeys[3].value = g_strdup("Ctrl Page_Down");
	// First Page
	pagekeys[4].name = g_strdup("first_tab_key");
	pagekeys[4].value = g_strdup("Ctrl Home");
	// Last Page
	pagekeys[5].name = g_strdup("last_tab_key");
	pagekeys[5].value = g_strdup("Ctrl End");
	
	GKeyFile* keyfile = g_key_file_new();
	gchar* path = g_strdup_printf("%s/%s", g_get_user_config_dir(), cfgfile);
	GError* error = NULL;
	
	if ( g_key_file_load_from_file(keyfile, path, G_KEY_FILE_NONE, &error))
	{
		gchar* value = NULL;

		value = g_key_file_get_value(keyfile, "main", "page_name", NULL);
		if (value) page_name = value;

		value = g_key_file_get_value(keyfile, "main", "foreground_color", NULL);
		if (value) foreground_color = value;

		value = g_key_file_get_value(keyfile, "main", "background_color", NULL);
		if (value) background_color = value;

		value = g_key_file_get_value(keyfile, "main", "font_name", NULL);
		if (value) font_name = value;

		value = g_key_file_get_value(keyfile, "main", "screen_with", NULL);
		if (value) screen_with = atoi(value);

		value = g_key_file_get_value(keyfile, "main", "screen_height", NULL);
		if (value) screen_height = atoi(value);

		value = g_key_file_get_value(keyfile, "main", "transparent_background", NULL);
		if (value) transparent_background = (gboolean)(atoi(value));

		value = g_key_file_get_value(keyfile, "main", "background_saturation", NULL);
		if (value && atof(value) < 1 && atof(value) > 0) background_saturation = atof(value);

		value = g_key_file_get_value(keyfile, "main", "word_chars", NULL);
		if (value) word_chars = value;

		value = g_key_file_get_value(keyfile, "main", "scrollback_lines", NULL);
		if (value) scrollback_lines = atoi(value);

		value = g_key_file_get_value(keyfile, "main", "locales_list", NULL);
		if (value) locales_list = value;

		struct KeyValue tempkey;
		gint i=0;
//		g_debug("Key Value: Shift=%x, NumLock=%x, Control=%x, Mod1=%x, Mod2=%x, Mod3=%x, Mod4=%x, Mod5=%x\n",
//				 GDK_SHIFT_MASK, GDK_LOCK_MASK, GDK_CONTROL_MASK, GDK_MOD1_MASK, GDK_MOD2_MASK, GDK_MOD3_MASK, 
//				 GDK_MOD4_MASK, GDK_MOD5_MASK);
		for (i=0; i<KEYS; i++)
		{
			tempkey.ukey=0;tempkey.lkey=0;tempkey.mods=0;
			value = g_key_file_get_value(keyfile, "key", pagekeys[i].name, NULL);
			if (value)
			{
//				g_debug("Got %s key = %s form config file.\n", pagekeys[i].name, value);
				accelerator_parse (value, &(tempkey.ukey), &(tempkey.lkey), &(tempkey.mods));
				if (tempkey.ukey)
				{
					// use the user's custom key
					pagekeys[i].ukey = tempkey.ukey;
					pagekeys[i].lkey = tempkey.lkey;
					pagekeys[i].mods = tempkey.mods;
//					g_debug("Got key %x(%s), %x(%s), mods = %d.\n",
//					gdk_keyval_name(pagekeys[i].ukey), gdk_keyval_name(pagekeys[i].ukey),
//					gdk_keyval_name(pagekeys[i].lkey), gdk_keyval_name(pagekeys[i].lkey),
//					pagekeys[i].mods);
				}	
			}
			if (! tempkey.ukey)
			{
				// use the default key
				accelerator_parse (pagekeys[i].value, &(pagekeys[i].ukey), &(pagekeys[i].lkey), &(pagekeys[i].mods));
//				g_debug("Use default key %x(%s), %x(%s), mods = %d.\n",
//				gdk_keyval_name(pagekeys[i].ukey), gdk_keyval_name(pagekeys[i].ukey),
//				gdk_keyval_name(pagekeys[i].lkey) , gdk_keyval_name(pagekeys[i].lkey),
//				pagekeys[i].mods);
			}
		}
	}
	else
	{
		g_debug("Warnning: the config file %s is invalid: %s\n", path, error->message);
		g_error_free (error);
	}

	supported_locales = g_strsplit_set(locales_list, " ,", 0);
	// if supported_locales is unset, don't show menu
	if (supported_locales!=NULL)
		show_menu = TRUE;
	
	// get the default locale from environment
	get_default_locale();

	g_key_file_free(keyfile);
	g_free(path);
}

void get_default_locale()
{
	// the idea of these codes is came from GCIN. Pay homage to Edward Liu!
	char* LC_CTYPE = (char*)g_getenv("LC_CTYPE");
	char* LC_ALL = (char*)g_getenv("LC_ALL");
	char* LANG = (char*)g_getenv("LANG");

	if ( (!LC_CTYPE) && LANG)
		LC_CTYPE = LANG;

	if (LC_ALL)
		LC_CTYPE = LC_ALL;

	if (!LC_CTYPE)
		LC_CTYPE = "UTF-8";

	default_locale = g_strdup(LC_CTYPE);
}

void init_new_page(GtkWidget *vtebox)
{
	// set font/background colors
	GdkColor color;
	vte_terminal_set_default_colors(VTE_TERMINAL(vtebox));
	gdk_color_parse(foreground_color, &color);
	vte_terminal_set_color_foreground(VTE_TERMINAL(vtebox), &color);
	gdk_color_parse(background_color, &color);
	vte_terminal_set_color_background(VTE_TERMINAL(vtebox), &color);

	// set font & terminal size
	vte_terminal_set_font_from_string(VTE_TERMINAL(vtebox), font_name);
	vte_terminal_set_size(VTE_TERMINAL(vtebox), screen_with, screen_height);

	// set transparent
	if (transparent_background)
	{
		vte_terminal_set_background_transparent(VTE_TERMINAL(vtebox), TRUE);
		vte_terminal_set_background_saturation(VTE_TERMINAL(vtebox), background_saturation);
	}

	// other settings
	vte_terminal_set_word_chars(VTE_TERMINAL(vtebox), word_chars);
	vte_terminal_set_scrollback_lines(VTE_TERMINAL(vtebox), scrollback_lines);

	// some fixed parameter
	vte_terminal_set_scroll_on_output(VTE_TERMINAL(vtebox), FALSE);
	vte_terminal_set_scroll_on_keystroke(VTE_TERMINAL(vtebox), TRUE);
}

void accelerator_parse (const gchar* key_name, guint *ukey, guint *lkey, guint *mods)
{
	// key_name example: "Ctrl+Shift Home"

	gchar** keynames;
	gchar** functions;
	gchar** keys;
	gint i=0;
	gint tempmods=0;
	gint tempkey=0;

	// got the Function key first
	keynames = g_strsplit_set(key_name, " ", 0);
	// split in to "Ctrl+Shift" and "Home" now...
	if (keynames)
	{
		functions = g_strsplit_set(keynames[0], "+", 0);
		// got the function key ("Ctrl+Shift" for example)
		while (functions[i]!=NULL)
		{
//			g_debug("dealing %s...\n", functions[i]);
			if ( ! g_ascii_strcasecmp (functions[i], "Ctrl"))
				tempmods |= GDK_CONTROL_MASK;
			else if ( ! g_ascii_strcasecmp (functions[i], "Shift"))
				tempmods |= GDK_SHIFT_MASK;
			else if ( ! g_ascii_strcasecmp (functions[i], "Alt"))
				tempmods |= GDK_MOD1_MASK;
			else if ( ! g_ascii_strcasecmp (functions[i], "Mod1"))
				tempmods |= GDK_MOD1_MASK;
//			else if ( ! g_ascii_strcasecmp (functions[i], "Mod2"))
//				tempmods |= GDK_MOD2_MASK;
			else if ( ! g_ascii_strcasecmp (functions[i], "Mod3"))
				tempmods |= GDK_MOD3_MASK;
			else if ( ! g_ascii_strcasecmp (functions[i], "Mod4"))
				tempmods |= GDK_MOD4_MASK;
			else if ( ! g_ascii_strcasecmp (functions[i], "Mod5"))
				tempmods |= GDK_MOD5_MASK;
			i++;
		}
//		g_debug("masks = %x\n", tempmods);
		if (! tempmods)
			g_debug("ERROR: No Function Key found!\n");
		
		if (keynames[1])
		{
			// got the key ("Home" for example)
			tempkey = gdk_keyval_from_name(keynames[1]);
			if (tempkey == GDK_VoidSymbol)
			{
				// not a valid key
				g_debug("ERROR: %s is not a valid key!\n", keynames[1]);
			}
			else
			{
//				g_debug("%s (%s, value %x) is a valid key!\n",
//						 keynames[1], gdk_keyval_name(tempkey), tempkey);
				*ukey=tempkey;
				// Find the Upper and Lower key value
				if ((tempkey>=GDK_A) && (tempkey<=GDK_Z))
					*lkey=tempkey-GDK_A+GDK_a;
				else if ((tempkey>=GDK_a) && (tempkey<=GDK_z))
					*lkey=tempkey-GDK_a+GDK_A;
				else
					*lkey=0;
				// suppose that CapsLock & NumLock always on
				*mods=(tempmods|GDK_LOCK_MASK)&0xf;
			}
		}
		else
			g_debug("ERROR: No Work Key found!\n");
	}
	else
	{
		// not a valid key
		g_debug("ERROR: %s is not a valid key at all!\n", keynames[1]);
	}
}
