/*
 * Copyright (c) 2008 Lu, Chao-Ming (Tetralet).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "main.h"

#define PACKAGE "lilyterm"

extern gboolean show_menu;
extern struct KeyValue pagekeys[KEYS];

GtkWidget *window;
GtkWidget *notebook;

int main( int   argc,
		  char *argv[] )
{
	gtk_init(&argc, &argv);

	// get user settings
	get_user_settings();

	// create main window
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), PACKAGE);
//	GdkPixbuf *icon = gdk_pixbuf_new_from_file(ICONDIR G_DIR_SEPARATOR_S PACKAGE ".png", NULL);
//	gtk_window_set_icon(GTK_WINDOW(window), icon);
//	if (icon)
//		g_object_unref(icon);
//	gtk_window_set_default_icon_name(PACKAGE);

	// close application if [Close Button] clicked
	g_signal_connect(G_OBJECT(window), "delete_event",
					  G_CALLBACK(gtk_main_quit), NULL);
	// if function key pressed
	g_signal_connect(G_OBJECT(window), "key-press-event", 
					  G_CALLBACK(key_press), NULL);

	// create notebook
	notebook = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	gtk_container_add(GTK_CONTAINER(window), notebook);

	// add a new page to notebook
	add_page();

	// create menu for switching locales
	if (show_menu)
		create_menu();

	// finish!
	gtk_widget_show_all(window);
	gtk_main();

	return 0;
}

static gboolean key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	if (event->state & ALL_ACCELS_MASK)
	{
//		g_debug("Got the function key: %s (status: %X, check: %X)\n",gdk_keyval_name(event->keyval), event->state, (event->state|GDK_LOCK_MASK)&0xf);
		gint i;
		for (i=0;i<KEYS;i++)
		{
//			g_debug("Checking... key: %s (status: %X)\n",gdk_keyval_name(pagekeys[i].ukey), event->state|GDK_LOCK_MASK);
			if (((((event->state|GDK_LOCK_MASK)&0xf)==pagekeys[i].mods) && (event->keyval==pagekeys[i].ukey)) ||
				  ((pagekeys[i].lkey) && (((event->state|GDK_LOCK_MASK)&0xf)==pagekeys[i].mods) && (event->keyval==pagekeys[i].lkey)))
			{
				// deal the function key
				deal_key_press(i);
				return TRUE;
			}
		}
	}
	return FALSE;
}

// Default function key:
// [0] <Ctrl><Insert> = New Page
// [1] <Ctrl><Delete> = Close Page
// [2] <Ctrl><PageUp> = Pre Page
// [3] <Ctrl><PageDown> = Next Page
// [4] <Ctrl><Home> = First Page
// [5] <Ctrl><End> = Last Page
//     <Shift><Insert> to pase clipboard.
static void deal_key_press(gint type)
{
	switch (type)
	{
		case 0:
		{
			// add a new page
			add_page();
			break;
		}
		case 1:
		{
			// close page
			close_page (NULL,NULL,NULL);
			break;
		}
		case 2:
		{
			// switch to pre page
			gint current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
			if (current_page)
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page - 1);
			else
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 
											  gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) - 1);
			break;
		}
		case 3:
		{
			// switch to next page
			gint current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
			if (current_page == (gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) - 1))
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
			else
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page + 1);
			break;
		}
		case 4:
		{
			// switch to first page
			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
			break;
		}
		case 5:
		{
			// switch to last page
			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook),
										  gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) - 1);
			break;
		}
	}
}

