/*
 * Copyright (c) 2008 Lu, Chao-Ming (Tetralet).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "notebook.h"
extern gchar *page_name;
extern gboolean show_menu;
extern VteTerminal *current_vtebox;
extern GtkWidget *locale_menu;

extern GtkWidget *window;
extern GtkWidget *notebook;

void add_page()
{
	// the component of a single page
	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget *vtebox;
	GtkWidget *scrollbar;
	gchar *encoding;
	pid_t pid;
	gint current_page;

	label = gtk_label_new(page_name);
	hbox = gtk_hbox_new(FALSE, 0);

	// create vtebox
	vtebox = vte_terminal_new();
	init_new_page(vtebox);
	gtk_box_pack_start(GTK_BOX(hbox), vtebox, TRUE, TRUE, 0);
	// the close page event
	g_signal_connect(G_OBJECT(vtebox), "child_exited",
					 G_CALLBACK(close_page), NULL);
#ifdef IM_PATCH
	// a dirty fix for IMs, (like gcin, xcin, oxim...)
	g_signal_connect(G_OBJECT(vtebox), "key-press-event",
					 G_CALLBACK(vtebox_key_press), VTE_TERMINAL(vtebox));
#endif
	if (show_menu)
		// show the LOCALE Selection menu
		g_signal_connect(G_OBJECT(vtebox), "button-press-event",
						 G_CALLBACK(vtebox_button_press), vtebox);

	// scrollbar
	scrollbar = gtk_vscrollbar_new(vte_terminal_get_adjustment(VTE_TERMINAL(vtebox)));
	gtk_box_pack_start(GTK_BOX(hbox), scrollbar, FALSE, FALSE, 0);

	// add the new page to notebook
	current_page = gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, label);
	gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(notebook), hbox,TRUE);
	gtk_notebook_set_tab_detachable(GTK_NOTEBOOK(notebook), hbox,FALSE);
	g_object_set(gtk_widget_get_parent(label), "can-focus", FALSE, NULL);

	// show the page bar if necessary
	if (current_page == 1)
		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), TRUE);

	// TODO: Use the current Page's PWD
	pid = vte_terminal_fork_command(VTE_TERMINAL(vtebox),
									g_getenv("SHELL"), NULL, NULL,
									g_getenv("PWD"), TRUE, TRUE,TRUE);
	
	// finish!
	gtk_widget_show_all(notebook);
	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page);
	gtk_window_set_focus(GTK_WINDOW(window), vtebox);
}

void close_page (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	gint totalpage=gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));

	// if it is the last page, exit application!
	if (totalpage == 1)
		gtk_main_quit();
	
	// remove current page
	gtk_notebook_remove_page(GTK_NOTEBOOK(notebook),
							 gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)));
	
	// hide the page bar if necessary
	if (totalpage == 2)
	{
		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
		// FIXME: Yes, just set it to 1 x 1, 
		// and the window will be resized to correct geometry automatically.
		gtk_window_resize(GTK_WINDOW(window), 1, 1);
	}
}

#ifdef IM_PATCH
static gboolean vtebox_key_press(GtkWidget *widget, GdkEventKey *event, VteTerminal *vtebox)
{
	// launch IMs to deal the inputs first
	return  (GTK_WIDGET_REALIZED(vtebox) && gtk_im_context_filter_keypress(vtebox->pvt->im_context, event));
}
#endif

gboolean vtebox_button_press(GtkWidget *widget, GdkEventButton *event, VteTerminal *vtebox)
{
	if (event->button == 3)
	{
		gtk_menu_popup(GTK_MENU(locale_menu), NULL, NULL, NULL, NULL, 3, (guint32)gtk_get_current_event_time());
		// I don't like it though...
		current_vtebox = VTE_TERMINAL(vtebox);
		return TRUE;
	}
	return FALSE;
}

